//modified by Jing Yang after 7/6/2001
////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool Version 4.0 - A program for Multivariate Data Display
//
//  Copyright 1994-1999, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//      Matthew O. Ward, Allen R. Martin, Ying-Huey Fua
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~xmdv
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  XmdvTool Development Team: <matt@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //

#ifndef OKCDATA_H
#define OKCDATA_H


#include <xmdv/ARR.h>
#include <xmdv/RGB.h>

#ifdef USE_META
struct IMeta;
struct Tcl_Interp;
#endif

struct NominalField
{
    int range_min, range_max;
    ARR<char*> values;

    const char *MapValueToString(int what);
};

//this structure contains all the information for an okc dataset and the functions to manipulate it
struct okcdata
{
    public:
        okcdata();
		
        bool Read(const char *fname);
        void Read2();
		void WriteBrushed(const char *fname);
        void GetAllData(ARR<double> &data, int line);
        void GetData(ARR<double> &data, int line);
        void SetFieldOnOff(int i, bool state);
		void ReorderDimension(int which, int forward);
		void Placement_Sort_Glyph(int dimNumber, bool increase);
		void Placement_Sort_Space_Glyph(int dimNumber, bool increase);
		void Placement_Original_Space_Glyph();
		void Placement_Original_Glyph();
		void Placement_DataDriven_Glyph(int XdimNumber, int YdimNumber);
        void AddBuffer();
        void GetPrin0(double* &prin0);
		void Placement_Derived_Glyph(int XdimNumber, int YdimNumber);
		void Placement_Shift_Glyph();
        void RemapAllData();
        void CountDims();
        void MapDataWithDimClusterExtends(ARR<double> all_data, ARR<double> &mean, ARR<double> &max, ARR<double> &min);
        void GetDataWithDimClusterExtends(ARR<double> &mean, ARR<double> &max, ARR<double> &min, int line);
		
        //shiping
        void setNumberOfBins(int which, int num);

		template <class T> void MapData( ARR<T> all_data, ARR<T> &data)
		{
			ARR<T> temp(all_dims);
			int i, loopi;
			for (loopi=0; loopi<all_dims; loopi++)
			{
				i = map_file_to_buf[loopi];
				temp[i] = all_data[loopi];
			}

			ARR<T> temp1(all_dims);
			int count = 0;
			for (i=0;i<all_dims;i++)
			{
				if (on[i]) 
				{
					temp1[count] = temp[i];
					count++;
				}
			}

			if ( (dim_reducer != NULL) && (dim_reducer->status == APPLY) )
			{
				dim_reducer->MapData(temp1, data);        
			}
			else
			{
				for (i=0;i<count;i++)
				{
					data[i] = temp1[i];
				}
			}
		}

        template <class T> void MapDataWithoutOnOff(ARR<T> all_data, ARR<T> &data)
		{
			int i, loopi;
			for (loopi=0; loopi<all_dims; loopi++)
			{
				i = map_file_to_buf[loopi];
				data[i] = all_data[loopi];
			}

		}

        template <class T> 
		void ReverseMapData( ARR<T> data, ARR<T> &all_data)
		{
			int i, file_i;
			int count = 0;
			for (i=0;i<all_dims;i++)
			{
				if (on[i]) 
				{
					file_i = map_buf_to_file[i];
					all_data[file_i] = data[count];
					count++;
				}
			}
		}
		
        void SaveBrushes();

		// Anil
		const char *MapValueToString(int dim, double value) const;
		const char *GetMin(int dim);
		const char *GetMax(int dim);

#ifdef USE_META
		IMeta *GetMeta() const;

		void ShowContents(Tcl_Interp *interp, int dim, double value) const;
		void CreateNewMeta();
#endif // USE_META
		// Anil
		
    public:
        //name of the okc file, from user input or default
        char* datafile;                 
        
        ////////////////////////////////////////////////////
        // Original data read from file. 
        //The order of the dimensions could be different from what's 
        //in the file since if reorder is performed, 
        //the file will be re-read according to the new order. 
        //
        
        //number of data points contained in the okc file
        int data_size;                  
        //number of origional dimensions contained in the okc file 
        int all_dims;                   
        //size of each of these two arrays is the above int all_dims. 
        //all_dim_min[k] is the mininum of the kth dimensional values 
        //of all the data points. 
        //all_dim_max[k] is the maximum of the kth dimensional values
        //of all the data points.
        //some validity checking and adjusting are performed on 
        //all_dim_min and all_dim_max before they are used.
        ARR<double> all_dim_min, all_dim_max;  
        //they are copies of all_dim_min and all_dim_max, 
        //to be used in the hierarchical display.
        ARR<double> cg_dim_min, cg_dim_max;
        //size of this array is int all_dims. 
        //all_cardinality[k] is the cardinality of the kth dimension
        //related to number of bins in dimension stacking. 
        ARR<int> all_cardinality;
        //size of this array is int all_dims. 
        //all_names[k] is the name of the kth dimension. 
        ARR<char*> all_names;
        //size of this array is int all_dims. 
        //all_nominal_fields[k] is the nominal files of the kth dimension. 
        ARR<NominalField*> all_nominal_fields;
        //buffer in main memory for storing all the data points in full dimensions
        ARR<double> data_buf;           

        ///////////////////////////////////////////////////////////////////////////
        // Active dimensions in dataset
        //if a dimension is in "off" status in the dimension on/off/reorder dialog, 
        //it will not appear here.
        //the order of the dimensions follows their order in the dimension on/off/reorder dialog.
        //
        //the number of all currently "on" dimensions
        int dims;
        //size of this array is int all_dims. 
        //on[k] records the current "on/off" status of the kth dimension showed 
        //in the dimension on/off/reorder dialog.
        ARR<bool> on;
        //size of each of these two arrays should be int dims
        //org_dim_min[k] is the mininum of the kth "on" dimensional values of all the data points. 
        //org_dim_max[k] is the maximum of the kth "on" dimensional values of all the data points.
        ARR<double> org_dim_min, org_dim_max;
        //size of each of these two arrays should be int dims, but they are allocated all_dims size in 
        //the code. (all_dimes is larger than dims). 
        //dim_min[k] is the mininum of the kth "on" dimensional values on the screen
        //dim_max[k] is the maximum of the kth "on" dimensional values on the screen
        ARR<double> dim_min, dim_max;
        //size of the array is int dims
        //cardinality[k] is the cardinality of the kth "on" dimension 
        ARR<int> cardinality;
        //size of the array is int dims
        //names[k] is the name of the kth "on" dimension 
        ARR<char*> names;
        //size of the array is int dims
        //nominal_fields[k] is the nominal_field of the kth "on" dimension 
        ARR<NominalField*> nominal_fields;
        //siz of this array is int dims
        //axis_color[k] is the color of the kth dimension
        ARR<RGBt> axis_color;
        // Computed data -- Do not modify
        //
        // 1/dims
        double one_over_dims;

        ////////////////////////////////////////////////////////////////////
        //used for data display
        //
        enum mark_type { REGULAR, DISABLED, COLORED };
        //siz of this array is int data_size
        //mark[k] is the mark_type of the kth data point
        ARR<mark_type> mark;
        //siz of this array is int data_size
        //color[k] is the color of the kth data point
        ARR<RGBt> color;

        
        ////////////////////////////////////////////////////////////////////
		//used for reordering dimension
        //
        //size of the array is int all_dims
        //map_file_to_buf[k] records the current order of the kth dimension in the okc file
		ARR<int> map_file_to_buf;
        //size of the array is int all_dims
        //map_file_to_buf[k] records the current kth dimension's order in the okc file
        ARR<int> map_buf_to_file;

		/////////////////////////////////////////////////////////////////////
        //used for glyph placement
        //
        //size of each of these two arrays is int data_size
        //basex[k] is the x coordinate of the kth data point in the canvas
        //basey[k] is the y coordinate of the kth data point in the canvas
        double *basex, *basey;
        // Computed data
        //
        //blocksize is the maximum size of a glyph
        //halfblocksize is half of the maximum size of a glyph
		double blocksize, halfblocksize;
		//number of lines of glyphs in the canvas when it is placed one by one 
        //it is also the number of glyphs in each line when it is placed one by one
        int blockcnt;

        //start_jing_animate_brush
        double *brushed_time;
        bool animate_brush;
        //end_jing_animate_brush

		//Anil
#ifdef USE_META
		IMeta *meta;
		bool hasMeta;
#endif
		//Anil

};

#endif
